SystemOrganization addCategory: #'Phidget-Examples'!

Object subclass: #PhdExIfKitLED
	instanceVariableNames: 'ifKit running'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLED commentStamp: 'pt 7/6/2009 16:54' prior: 0!
This is an example for the usage of an InterfaceKit (ifKit).
We expect that there are LED connected to the digital output of the ifKit.

execute this in a workspace:
	led := PhdExIfKitLED new. "creates a new LEDExample"
	led start.				"starts the animation - opens a connection to the ifKit"
	led stop.				"stops the animation - closes the connection - recommended if you want to use this Phidget later. If the connection stays open, the Phidget can not be used somewhere else."!

!PhdExIfKitLED methodsFor: 'initialize' stamp: 'pt 6/30/2009 11:12'!
initialize

	super initialize.
	ifKit := PhdCrInterfaceKit new.! !

!PhdExIfKitLED methodsFor: 'events' stamp: 'fs 7/24/2009 15:28'!
process

	[running] whileTrue: [
		ifKit outputs size = 0 ifFalse: [
			self setLEDState].
		(Delay forMilliseconds: self waitTime) wait]! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:30'!
setLEDState

	self toggle: {(1 to: ifKit outputs size) atRandom}.! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:30'!
start

	ifKit openAny.
	running := true.
	[self process] fork.! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:30'!
stop

	running := false.
	self turnAllOff.
	ifKit close.! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:32'!
toggle: anArray
	"toggle state if needed for all led"
	
	anArray do: [: i |
		(ifKit outputs at: i) value: (ifKit outputs at: i) value not]! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:32'!
turnAllOff

	(1 to: ifKit outputs size) do: [:i | (ifKit outputs at: i) value: false].! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:33'!
turnAllOn

	(1 to: ifKit outputs size) do: [:i | (ifKit outputs at: i) value: true].! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:33'!
turnOff: anArray

	anArray do: [:i | (ifKit outputs at: i) value: false].! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:33'!
turnOn: anArray

	anArray do: [:i | (ifKit outputs at: i) value: true].! !

!PhdExIfKitLED methodsFor: 'accessing' stamp: 'fs 7/24/2009 15:27'!
waitTime

	^ 200! !

PhdExIfKitLED subclass: #PhdExIfKitLEDCircles
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDCircles methodsFor: 'initialize' stamp: 'fs 7/24/2009 15:38'!
initialize

	super initialize.
	state := #1.! !

!PhdExIfKitLEDCircles methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:38'!
setLEDState

	self turnAllOff.
	state =  1 ifTrue: [                           state :=  2. ^ self].
	state =  2 ifTrue: [self turnOn: {1. 4. 5. 8}. state :=  3. ^ self].
	state =  3 ifTrue: [self turnOn: {2. 3. 6. 7}. state :=  1. ^ self].! !

!PhdExIfKitLEDCircles methodsFor: 'accessing' stamp: 'fs 7/24/2009 15:38'!
waitTime

	^ 150! !

PhdExIfKitLED subclass: #PhdExIfKitLEDNothing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDNothing commentStamp: 'pt 7/6/2009 16:57' prior: 0!
Does nothing, but turn all LEDs on. This may be used for own experiments. Edit setLEDstate to define what happens every <self waitTime> milliseconds. (You may also edit my method waitTime ;))

This example is based on PhdExIfKitLED. Have a look on the comment of that class for further information.!

!PhdExIfKitLEDNothing methodsFor: 'toggle state' stamp: 'pt 7/1/2009 12:01'!
setLEDState
	self turnAllOn.! !

!PhdExIfKitLEDNothing methodsFor: 'accessing' stamp: 'fs 7/24/2009 15:38'!
waitTime

	^ 1000! !

PhdExIfKitLED subclass: #PhdExIfKitLEDSpiral
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDSpiral commentStamp: 'pt 7/6/2009 16:55' prior: 0!
Makes the LEDs flash in a spiral.

This class expects the LED ordered as the following:
    4
    3
8 7   6 5
    2
    1

This example is based on PhdExIfKitLED. Have a look on the comment of that class for further information.!

!PhdExIfKitLEDSpiral methodsFor: 'initialize' stamp: 'pt 6/30/2009 12:01'!
initialize
	super initialize.
	state := #1.! !

!PhdExIfKitLEDSpiral methodsFor: 'toggle state' stamp: 'pt 6/30/2009 12:05'!
setLEDState
	
	state =  1 ifTrue: [self turnOn: {1.}. state :=  2. ^ self].
	state =  2 ifTrue: [self turnOn: {8.}. state :=  3. ^ self].
	state =  3 ifTrue: [self turnOn: {4.}. state :=  4. ^ self].
	state =  4 ifTrue: [self turnOn: {5.}. state :=  5. ^ self].
	state =  5 ifTrue: [self turnOn: {2.}. state :=  6. ^ self].
	state =  6 ifTrue: [self turnOn: {7.}. state :=  7. ^ self].
	state =  7 ifTrue: [self turnOn: {3.}. state :=  8. ^ self].
	state =  8 ifTrue: [self turnOn: {6.}. state :=  9. ^ self].
	
	state =  9 ifTrue: [self turnOff: {1.}. state := 10. ^ self].
	state = 10 ifTrue: [self turnOff: {8.}. state := 11. ^ self].
	state = 11 ifTrue: [self turnOff: {4.}. state := 12. ^ self].
	state = 12 ifTrue: [self turnOff: {5.}. state := 13. ^ self].
	state = 13 ifTrue: [self turnOff: {2.}. state := 14. ^ self].
	state = 14 ifTrue: [self turnOff: {7.}. state := 15. ^ self].
	state = 15 ifTrue: [self turnOff: {3.}. state := 16. ^ self].
	state = 16 ifTrue: [self turnOff: {6.}. state :=  1. ^ self].! !

!PhdExIfKitLEDSpiral methodsFor: 'accessing' stamp: 'pt 6/30/2009 15:14'!
waitTime
	^ 150! !

PhdExIfKitLED subclass: #PhdExIfKitLEDToggle
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDToggle commentStamp: 'pt 7/6/2009 16:58' prior: 0!
Makes the two LED axes alternately flash.

This class expects the LED ordered as the following:
    4
    3
8 7   6 5
    2
    1

This example is based on PhdExIfKitLED. Have a look on the comment of that class for further information.!

!PhdExIfKitLEDToggle methodsFor: 'initialize' stamp: 'fs 7/24/2009 15:41'!
initialize

	super initialize.
	state := #1.! !

!PhdExIfKitLEDToggle methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:41'!
setLEDState

	self turnAllOff.
	state = #1 ifTrue: [self turnOn: {1. 2. 3. 4.}. state := 2. ^ self].
	state = #2 ifTrue: [self turnOn: {5. 6. 7. 8.}. state := 1. ^ self].! !

Object subclass: #PhdExListItem
	instanceVariableNames: 'nextLink value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExListItem commentStamp: 'pt 7/6/2009 16:46' prior: 0!
This is a class used in the PhidgetLab RFID example.
See PhdExRFIDPaint or PhdExTagMap for more information about the example.!

!PhdExListItem class methodsFor: 'as yet unclassified' stamp: 'lk 7/2/2009 22:36'!
new: anObject
	"create a new list item with given object as content."
	
	^ self new
		value: anObject.! !

!PhdExListItem methodsFor: 'accessing' stamp: 'lk 7/2/2009 21:52'!
nextLink

	^ nextLink! !

!PhdExListItem methodsFor: 'accessing' stamp: 'lk 7/2/2009 21:52'!
nextLink: anObject

	nextLink := anObject.! !

!PhdExListItem methodsFor: 'accessing' stamp: 'lk 7/2/2009 21:53'!
value: anObject

	value := anObject.! !

Object subclass: #PhdExLunarLander
	instanceVariableNames: 'motorControl interfaceKit leftRocket rightRocket started resetSemaphore stopping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/19/2009 22:44'!
allPhidgetsAttached

	^ motorControl isAttached and: [ interfaceKit isAttached ]! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/19/2009 14:08'!
closePhidgets

	motorControl close.
	interfaceKit close.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:07'!
controllersAreZero

	^ (interfaceKit sensors at: 5) value // 100 = 0 and: [(interfaceKit sensors at: 6) value // 100 = 0]! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 08:42'!
deleteStopRocketsOnDetachCallbacks

	motorControl removeActionsWithReceiver: self forEvent: #Detach.
	interfaceKit removeActionsWithReceiver: self forEvent: #Detach.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:03'!
deleteTryToResetOnAttachCallbacks

	motorControl on: #Attach do: [].
	interfaceKit on: #Attach do: [].! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 11:16'!
forkWaitForRocketResetProcess

	[	"both rockets must have been reset. Then trigger the event"
		resetSemaphore wait.
		resetSemaphore wait.
		self triggerEvent: #rocketsReset.
		self removeActionsWithReceiver: self forEvent: #rocketsReset.
	] fork.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:27'!
initialize

	super initialize.
	
	started := false.
	stopping := false.
	resetSemaphore := Semaphore new.
	
	motorControl := PhdCrMotorControl new.
	interfaceKit := PhdCrInterfaceKit new.
	
	leftRocket := PhdExLunarLanderRocket new.
	rightRocket := PhdExLunarLanderRocket new.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/19/2009 14:08'!
openPhidgets

	motorControl openAny.
	interfaceKit openAny.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:03'!
registerResetOnAttachEvents

	motorControl on: #Attach do: [ self tryToResetRocketsAndSignal ].
	interfaceKit on: #Attach do: [ self tryToResetRocketsAndSignal ].! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:04'!
registerRocketsResetEvent

	self
		when: #rocketsReset
		send: #waitToZeroController
		to: self.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 07:13'!
registerStopOnDetachCallbacks

	motorControl when: #Detach send: #stopRockets to: self.
	interfaceKit when: #Detach send: #stopRockets to: self.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 11:16'!
resetRocketsAndSignal

	[ leftRocket resetAndSignal: resetSemaphore ] fork.
	[ rightRocket resetAndSignal: resetSemaphore ] fork.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:19'!
setRocketData

	leftRocket
		motor: (motorControl motors at: 1);
		baseInput: (interfaceKit inputs at: 1);
		controlSensor: (interfaceKit sensors at: 5);
		ledOutputs: { interfaceKit outputs at: 6. interfaceKit outputs at: 5. interfaceKit outputs at: 4 };
		isInverted: true.

	rightRocket
		motor: (motorControl motors at: 2);
		baseInput: (interfaceKit inputs at: 2);
		controlSensor: (interfaceKit sensors at: 6);
		ledOutputs: { interfaceKit outputs at: 3. interfaceKit outputs at: 2. interfaceKit outputs at: 1 };
		isInverted: false.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:27'!
start
	"wait for attachment, when both phidgets are attached, reset the rockets.
	when finished with that, start the Lunar Lander game."

	stopping := false.
	self
		registerResetOnAttachEvents;
		registerRocketsResetEvent;
		forkWaitForRocketResetProcess;
		openPhidgets.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 11:16'!
startRockets

	started ifTrue: [ ^ self ].
	started := true.

	[ leftRocket fly ] fork.
	[ rightRocket fly ] fork.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:27'!
stop

	stopping := true.
	[resetSemaphore isEmpty] whileFalse: [ resetSemaphore terminateProcess ].
	self deleteStopRocketsOnDetachCallbacks.
	
	self stopRockets.
	self closePhidgets.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/19/2009 16:39'!
stopRockets

	leftRocket stop.
	rightRocket stop.
	started := false.! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 08:49'!
tryToResetRocketsAndSignal

	self allPhidgetsAttached ifTrue: [
		self
			deleteTryToResetOnAttachCallbacks;
			registerStopOnDetachCallbacks;
			setRocketData;
			resetRocketsAndSignal.
	].! !

!PhdExLunarLander methodsFor: 'as yet unclassified' stamp: 'lk 8/20/2009 12:28'!
waitToZeroController

	stopping ifTrue: [ ^ self ].

	self controllersAreZero
		ifTrue: [
			interfaceKit on: #SensorChange do: [].
			1 to: 6 do: [ :i | (interfaceKit outputs at: i) value: false ].
			self startRockets.
		]
		ifFalse: [
			1 to: 6 do: [ :i | (interfaceKit outputs at: i) value: true ].
			interfaceKit on: #SensorChange do: [ self waitToZeroController. ].
		].! !

Object subclass: #PhdExLunarLanderRocket
	instanceVariableNames: 'motor baseInput controlSensor ledOutputs isInverted game state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExLunarLanderRocket class methodsFor: 'as yet unclassified' stamp: 'lk 8/19/2009 23:38'!
newWith: aLunarLanderGame

	^ self new
		game: aLunarLanderGame! !

!PhdExLunarLanderRocket methodsFor: 'accessing' stamp: 'lk 8/19/2009 19:22'!
baseInput: aPhdCrCompDigitalInput

	baseInput := aPhdCrCompDigitalInput.! !

!PhdExLunarLanderRocket methodsFor: 'resetting' stamp: 'lk 8/20/2009 12:35'!
blinkOnReset

	| delay ledState |
	delay := Delay forMilliseconds: 400.
	ledState := true.

	[ self isResetting ] whileTrue: [
		ledOutputs do: [ :led | led value: ledState ].
		ledState := ledState not.
		delay wait.
	].! !

!PhdExLunarLanderRocket methodsFor: 'accessing' stamp: 'lk 8/19/2009 19:21'!
controlSensor: aPhdCrCompAnalogSensor

	controlSensor := aPhdCrCompAnalogSensor.! !

!PhdExLunarLanderRocket methodsFor: 'flying' stamp: 'lk 8/20/2009 11:00'!
explodeLimit

	^ -15! !

!PhdExLunarLanderRocket methodsFor: 'flying' stamp: 'lk 8/20/2009 00:08'!
fly

	| delay sensorValue velocity |
	
	state := #flying.
	velocity := 0.
	motor velocity: velocity.
	delay := Delay forMilliseconds: 200.
	
	[ self isFlying and: [ self isOnTheGround not ] ] whileTrue: [
	
		"motor and gravity"
		sensorValue := controlSensor value.
		isInverted
			ifTrue: [ velocity := velocity - (sensorValue // 100) - self gravity ]
			ifFalse: [ velocity := velocity + (sensorValue // 100) + self gravity ].
		
		motor velocity: velocity.
		self setLedStateForVelocity: velocity.
		
		delay wait.
	].

	self stop.! !

!PhdExLunarLanderRocket methodsFor: 'accessing' stamp: 'lk 8/19/2009 23:39'!
game: aLunarLanderGame

	game := aLunarLanderGame.! !

!PhdExLunarLanderRocket methodsFor: 'flying' stamp: 'lk 8/19/2009 16:11'!
gravity

	^ -5! !

!PhdExLunarLanderRocket methodsFor: 'initialization' stamp: 'lk 8/19/2009 23:52'!
initialize

	super initialize.
	
	state := #stopped. "#flying #resetting #won #lost"! !

!PhdExLunarLanderRocket methodsFor: 'states' stamp: 'lk 8/19/2009 23:46'!
isFlying

	^ state = #flying! !

!PhdExLunarLanderRocket methodsFor: 'accessing' stamp: 'lk 8/19/2009 16:09'!
isInverted: aBoolean

	isInverted := aBoolean.! !

!PhdExLunarLanderRocket methodsFor: 'accessing' stamp: 'lk 8/19/2009 19:25'!
isOnTheGround

	^ baseInput value! !

!PhdExLunarLanderRocket methodsFor: 'states' stamp: 'lk 8/19/2009 23:46'!
isResetting

	^ state = #resetting! !

!PhdExLunarLanderRocket methodsFor: 'states' stamp: 'lk 8/19/2009 23:41'!
isStopped

	^ state = #stopped | state = #won | state = #lost! !

!PhdExLunarLanderRocket methodsFor: 'accessing' stamp: 'lk 8/20/2009 00:01'!
ledOutputs: aCollection

	ledOutputs := aCollection.! !

!PhdExLunarLanderRocket methodsFor: 'flying' stamp: 'lk 8/20/2009 11:00'!
maxUncertaintyLimit

	^ -40! !

!PhdExLunarLanderRocket methodsFor: 'flying' stamp: 'lk 8/20/2009 10:59'!
minUncertaintyLimit

	^ -10! !

!PhdExLunarLanderRocket methodsFor: 'accessing' stamp: 'lk 8/19/2009 19:22'!
motor: aPhdCrCompMotorControl

	motor := aPhdCrCompMotorControl.! !

!PhdExLunarLanderRocket methodsFor: 'resetting' stamp: 'lk 8/20/2009 10:56'!
resetAndSignal: aSemaphore

	| delay velocityUp velocityDown |

	self stop.
	state := #resetting.
	
	"blink while resetting =)"
	[ self blinkOnReset ] fork.
	
	delay := Delay forMilliseconds: 200.
	isInverted
		ifTrue: [ velocityUp := -50. velocityDown := 50 ]
		ifFalse: [ velocityUp := 50. velocityDown := -50 ].
	
	"if already on the ground, wind up until not on ground any more to prevent drooping ropes"
	(self isResetting and: [self isOnTheGround]) ifTrue: [
		motor velocity: velocityUp.
		[ self isResetting and: [ self isOnTheGround ] ] whileTrue: [ delay wait ].
		motor velocity: 0.
	].
	
	"go down to the ground"
	(self isResetting and: [self isOnTheGround]) ifFalse: [
		motor velocity: velocityDown.
		[ self isResetting and: [ self isOnTheGround not ] ] whileTrue: [ delay wait ].
		motor velocity: 0.
	].

	"wind up"
	motor velocity: velocityUp.
	47 timesRepeat: [
		self isResetting ifFalse: [ self stop. aSemaphore signal. ^ self ].
		delay wait
	].
	
	self stop.
	aSemaphore signal.! !

!PhdExLunarLanderRocket methodsFor: 'flying' stamp: 'lk 8/20/2009 11:48'!
setLedStateForVelocity: velocity

	| velocityToCheck |
	
	velocityToCheck := isInverted
		ifTrue: [ velocity negated ]
		ifFalse: [ velocity ]. 

	velocityToCheck >= self minUncertaintyLimit ifTrue: [
		(ledOutputs at: 1) value: true.
		(ledOutputs at: 2) value: false.
		(ledOutputs at: 3) value: false.
		^ self
	].
	velocityToCheck <= self maxUncertaintyLimit ifTrue: [
		(ledOutputs at: 1) value: false.
		(ledOutputs at: 2) value: false.
		(ledOutputs at: 3) value: true.
		^ self
	].
	"else"
	(ledOutputs at: 1) value: false.
	(ledOutputs at: 2) value: true.
	(ledOutputs at: 3) value: false.! !

!PhdExLunarLanderRocket methodsFor: 'states' stamp: 'lk 8/19/2009 23:45'!
stop

	motor ifNotNil: [ motor velocity: 0 ].
	state := #stopped.! !

Object subclass: #PhdExSqueakLogoLCD
	instanceVariableNames: 'lcdPhidget'
	classVariableNames: 'CustomCharCollection'
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExSqueakLogoLCD class methodsFor: 'as yet unclassified' stamp: 'fs 8/13/2009 19:54'!
customCharCollection

	CustomCharCollection ifNotNil: [ ^ CustomCharCollection].
	
	CustomCharCollection := Array new: 8.

	^ CustomCharCollection
		at: 1 put: (PhdCrCustomChar newWith: 542284 and: 4360);
		at: 2 put: (PhdCrCustomChar newWith: 0 and: 420684);
		at: 3 put: (PhdCrCustomChar newWith: 0 and: 210342);
		at: 4 put: (PhdCrCustomChar newWith: 50470 and: 4162);
		at: 5 put: (PhdCrCustomChar newWith: 397063 and: 8339);
		at: 6 put: (PhdCrCustomChar newWith: 60720 and: 24);
		at: 7 put: (PhdCrCustomChar newWith: 552513 and: 3);
		at: 8 put: (PhdCrCustomChar newWith: 221308 and: 2201)
	! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/12/2009 22:25'!
initialize

	super initialize.
	
	lcdPhidget := PhdCrTextLCD new.! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/13/2009 19:53'!
setCustomChars
		
	lcdPhidget textLCD setAllCustomChars: self class customCharCollection.! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/13/2009 19:39'!
setText

	lcdPhidget textLCD
		displayString: 
			PhdCrCustomChar first, 
			PhdCrCustomChar second, 
			PhdCrCustomChar third, 
			PhdCrCustomChar fourth,
			'  I''m a' 
		inRow: 1;
		displayString: 
			PhdCrCustomChar fifth, 
			PhdCrCustomChar sixth, 
			PhdCrCustomChar seventh, 
			PhdCrCustomChar eighth,
			'  Squeeeeaker!!' 
		inRow: 2.! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/12/2009 22:39'!
start

	lcdPhidget openAny.
	lcdPhidget on: #Attach do: [
		self 
			setCustomChars;
			setText.
	].! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/12/2009 22:34'!
stop

	lcdPhidget close.! !

Object subclass: #PhdExStarTrekLCD
	instanceVariableNames: 'running lcdPhidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExStarTrekLCD commentStamp: 'pt 7/15/2009 08:31' prior: 0!
This is an example for the usage of a Text LCD Phidget (usually bundled with an Interface Kit).

Execute the following line after you've attached the LCD Phidget.

"Create a new example object."
lcd := PhdExTextLCD new.

"Start the demo - this opens a connection to the phidget"
lcd start.

"Stop the demo - this closes the connection to the phidget"
lcd stop.!

!PhdExStarTrekLCD methodsFor: 'initialize' stamp: 'pt 7/15/2009 07:50'!
initialize

	super initialize.
	running := false.
	lcdPhidget := PhdCrTextLCD new.! !

!PhdExStarTrekLCD methodsFor: 'events' stamp: 'fs 7/24/2009 15:52'!
starTrekProcess
	"Let some stars slide along the display.
	 Insiders know where to find it in Star Treck ;)"

	| t |
	t := '          '.
	[ running ] whileTrue: [
		10 to: 1 by: -2 do: [ :i | | k |
			t at: i put: $*; at: i - 1 put: $*.
			k := t , t reverse.
			lcdPhidget textLCD
				displayString: k inRow: 1;
				displayString: k	inRow: 2.
			(Delay forMilliseconds: 500) wait.
			t at: i put: Character space; at: i - 1 put: Character space ] ].! !

!PhdExStarTrekLCD methodsFor: 'events' stamp: 'fs 7/24/2009 15:51'!
start

	running := true.
	lcdPhidget openAny.
	[ self starTrekProcess ] fork! !

!PhdExStarTrekLCD methodsFor: 'events' stamp: 'fs 7/24/2009 15:51'!
stop

	running := false.
	lcdPhidget close.! !

Object subclass: #PhdExTagMap
	instanceVariableNames: 'sensors tags knownTagHandler newTagHandler tagLostHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExTagMap commentStamp: 'pt 7/6/2009 16:53' prior: 0!
This is an example about how to use the RFID Phidget from Squeak with PhidgetLab.
This one is more generci than PhdExRFIDPaint and need further programming.

If you give an instance of this class a sensor from an RFID Phidget (PhdCrRFID) it associates an object to each RFID tag. You can specefy the behavior for new/known tags. You might also specify what happens if a tag was found/lost.
To make an instance forget known tags, execute: self forgetTags.

For an example look at PhdExRFIDPaint.!

PhdExTagMap subclass: #PhdExRFIDPaint
	instanceVariableNames: 'rfidPhidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExRFIDPaint commentStamp: 'pt 7/6/2009 16:54' prior: 0!
This is an example about how to use the Phidget RFID with Squeak/PhidgetLab.

Execute the following in a workspace:
map := PhdExRFIDPaint new. "opens a new instance of this example"
map start. "starts the 'game' and opens a connection to the Phidget"
map stop. "closes the connection - recommended if you want to use this Phidget later. If the connection stays open, the Phidget can not be used somewhere else."!

!PhdExRFIDPaint methodsFor: 'events' stamp: 'fs 7/24/2009 15:47'!
attach
	"Enables the antenna, if I was attached."
	
	rfidPhidget rfidSensor isAntennaOn: true! !

!PhdExRFIDPaint methodsFor: 'initialization' stamp: 'pt 7/6/2009 16:40'!
initialize

	super initialize.
	rfidPhidget := PhdCrRFID new.
	
	"set own handlers that cannot be overwritten."
	newTagHandler := [ :tagString |
		SketchMorph openEditor ].
	
	knownTagHandler := [ :tagString :oldInfo | 
		oldInfo
			openInWorld;
			visible: true;
			yourself ].
	
	tagLostHandler := [ :tagString :oldInfo | 
		oldInfo
			visible: false;
			yourself ].

	"register events"
	rfidPhidget on: #Attach	do: [ self attach ]! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
knownTagHandler

	^ knownTagHandler! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:45'!
knownTagHandler: aBlock
	"cannot be set in this subclass."! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
newTagHandler

	^ newTagHandler! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:45'!
newTagHandler: aBlock
	"cannot be set in this subclass."! !

!PhdExRFIDPaint methodsFor: 'as yet unclassified' stamp: 'pt 7/6/2009 16:39'!
start
	"opens a connection to an RFID phidget."
	
	rfidPhidget openAny.
	self
		addSensor: rfidPhidget rfidSensor;
		attach.! !

!PhdExRFIDPaint methodsFor: 'as yet unclassified' stamp: 'pt 7/3/2009 14:22'!
stop
	"close a connection to an RFID phidget."
	
	rfidPhidget close.! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
tagLostHandler

	^ tagLostHandler! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:45'!
tagLostHandler: aBlock
	"cannot be set in this subclass."! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:31'!
addSensor: anRFIDSensor

	sensors add: (PhdExListItem new: anRFIDSensor).
	anRFIDSensor on: #TagFound do: [ self tagFound: anRFIDSensor ].
	anRFIDSensor on: #TagLost do: [ self tagLost: anRFIDSensor ].! !

!PhdExTagMap methodsFor: 'initialization' stamp: 'pt 7/3/2009 14:40'!
forgetTags
	"clears my tag cache"
	
	tags := Dictionary new.! !

!PhdExTagMap methodsFor: 'initialization' stamp: 'lk 7/3/2009 11:58'!
initialize

	super initialize.
	
	sensors := LinkedList new.
	tags := Dictionary new.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:59'!
knownTagHandler

	knownTagHandler ifNil: [ 
		knownTagHandler := [ :tagString :oldInfo | oldInfo ] ].

	^ knownTagHandler! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/2/2009 23:40'!
knownTagHandler: aBlock
	"aBlock must take 1 argument: the String that represents the found tag we already know.
	When evaluated it must return an object to be associated with that tag."
	
	knownTagHandler := aBlock.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:59'!
newTagHandler

	newTagHandler ifNil: [
		newTagHandler := [ :tagString | tagString ] ].

	^ newTagHandler! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/2/2009 23:39'!
newTagHandler: aBlock
	"aBlock must take 1 argument: the String that represents the newly found tag..
	When evaluated it must return an object to be associated with that tag."
	
	newTagHandler := aBlock.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/2/2009 22:24'!
setTag: aString info: anObject

	tags at: aString asSymbol put: anObject.! !

!PhdExTagMap methodsFor: 'tag handler' stamp: 'lk 7/3/2009 12:05'!
tagFound: anRFIDSensor
	"anRFIDSensor is the sensor which found the tag"
	
	| tagString tagInfo |
	tagString := anRFIDSensor lastTag.
	
	tagInfo := (tags includesKey: tagString asSymbol)
		ifTrue:  [
			self knownTagHandler
				value: tagString
				value: (tags at: tagString asSymbol) ]
		ifFalse: [ self newTagHandler value: tagString ].
	
	self setTag: tagString info: tagInfo.! !

!PhdExTagMap methodsFor: 'tag handler' stamp: 'lk 7/3/2009 11:30'!
tagLost: anRFIDSensor
	"anRFIDSensor is the sensor which lost the tag"
	
	| tagString tagInfo |
	tagString := anRFIDSensor lastTag.
	
	tagInfo := (tags includesKey: tagString asSymbol)
		ifTrue:  [
			tagLostHandler
				value: tagString
				value: (tags at: tagString asSymbol) ]
		ifFalse: [ nil ].
	
	self setTag: tagString info: tagInfo.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
tagLostHandler

	tagLostHandler ifNil: [ 
		tagLostHandler := [ :tagString :oldInfo | oldInfo ] ].

	^ tagLostHandler! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:31'!
tagLostHandler: aBlock
	"aBlock must take 2 arguments: the String that represents the lost tag, and its currently set info.
	When evaluated it must return an object to be associated with that tag."
	
	tagLostHandler := aBlock.! !
